using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace WouldYouLikeMyInput
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //We'll use these just to give us a visual display so we know
        //our input code is working.
        SpriteFont mFont;
        string mAction;

        //The input class that will be making handling input a whole lot easier
        GameInput mGameInput;

        //Actions that will be defined for our "game" declared constants
        //so we don't have magic strings floating around and so we can use intellisense 
        //to access them
        const string ActionJump = "Jump";
        const string ActionExit = "Exit";
        const string ActionUp = "Up";
        const string ActionPause = "Pause";

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //Load our font we'll use to display our input information
            mFont = Content.Load<SpriteFont>("myFont");

            //Create a new instance of GameInput
            mGameInput = new GameInput();

            //Add keyboard and gamepad inputs for Jump
            mGameInput.AddGamePadInput(ActionJump, Buttons.A, true);
            mGameInput.AddKeyboardInput(ActionJump, Keys.A, true);
            mGameInput.AddKeyboardInput(ActionJump, Keys.Space, false);

            //Add keyboard and gamepad inputs for Pause
            mGameInput.AddGamePadInput(ActionPause, Buttons.Start, true);
            mGameInput.AddKeyboardInput(ActionPause, Keys.P, true);

            //Add keyboad and gamepad inputs for Up
            mGameInput.AddGamePadInput(ActionUp, Buttons.RightThumbstickUp, false);
            mGameInput.AddGamePadInput(ActionUp, Buttons.LeftThumbstickUp, false);
            mGameInput.AddGamePadInput(ActionUp, Buttons.DPadUp, false);
            mGameInput.AddKeyboardInput(ActionUp, Keys.Up, false);
            mGameInput.AddKeyboardInput(ActionUp, Keys.W, true);

            //Add keyboard and gamepad inputs for Exit
            mGameInput.AddGamePadInput(ActionExit, Buttons.Back, false);
            mGameInput.AddKeyboardInput(ActionExit, Keys.Escape, false);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            mGameInput.BeginUpdate();

            //Allows the game to exit
            if (mGameInput.IsPressed(ActionExit, PlayerIndex.One))
            {
                this.Exit();
            }

            mAction = string.Empty;

            if (mGameInput.IsPressed(ActionJump, PlayerIndex.One))
            {
                mAction = ActionJump;
            }

            if (mGameInput.IsPressed(ActionPause, PlayerIndex.One))
            {
                mAction = ActionPause;
            }

            if (mGameInput.IsPressed(ActionUp, PlayerIndex.One))
            {
                mAction = ActionUp;
            }

            mGameInput.EndUpdate();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            spriteBatch.DrawString(mFont, mAction, new Vector2(100, 100), Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
