﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace WouldYouLikeMyInput
{
    public class Input
    {
        Dictionary<Keys, bool> mKeyboard = new Dictionary<Keys, bool>();
        Dictionary<Buttons, bool> mGamepad = new Dictionary<Buttons, bool>();

        static public Dictionary<PlayerIndex, GamePadState> CurrentGamePadState = new Dictionary<PlayerIndex, GamePadState>();
        static public Dictionary<PlayerIndex, GamePadState> PreviousGamePadState = new Dictionary<PlayerIndex, GamePadState>();
        static public KeyboardState CurrentKeyboardState;
        static public KeyboardState PreviousKeyboardState;

        public Input()
        {
            if (CurrentGamePadState.Count == 0)
            {
                CurrentGamePadState.Add(PlayerIndex.One, GamePad.GetState(PlayerIndex.One));
                CurrentGamePadState.Add(PlayerIndex.Two, GamePad.GetState(PlayerIndex.Two));
                CurrentGamePadState.Add(PlayerIndex.Three, GamePad.GetState(PlayerIndex.Three));
                CurrentGamePadState.Add(PlayerIndex.Four, GamePad.GetState(PlayerIndex.Four));

                PreviousGamePadState.Add(PlayerIndex.One, GamePad.GetState(PlayerIndex.One));
                PreviousGamePadState.Add(PlayerIndex.Two, GamePad.GetState(PlayerIndex.Two));
                PreviousGamePadState.Add(PlayerIndex.Three, GamePad.GetState(PlayerIndex.Three));
                PreviousGamePadState.Add(PlayerIndex.Four, GamePad.GetState(PlayerIndex.Four));
            }
        }

        static public void BeginUpdate()
        {
            CurrentGamePadState[PlayerIndex.One] = GamePad.GetState(PlayerIndex.One);
            CurrentGamePadState[PlayerIndex.Two] = GamePad.GetState(PlayerIndex.Two);
            CurrentGamePadState[PlayerIndex.Three] = GamePad.GetState(PlayerIndex.Three);
            CurrentGamePadState[PlayerIndex.Four] = GamePad.GetState(PlayerIndex.Four);

            CurrentKeyboardState = Keyboard.GetState(PlayerIndex.One);
        }

        static public void EndUpdate()
        {
            PreviousGamePadState[PlayerIndex.One] = CurrentGamePadState[PlayerIndex.One];
            PreviousGamePadState[PlayerIndex.Two] = CurrentGamePadState[PlayerIndex.Two];
            PreviousGamePadState[PlayerIndex.Three] = CurrentGamePadState[PlayerIndex.Three];
            PreviousGamePadState[PlayerIndex.Four] = CurrentGamePadState[PlayerIndex.Four];

            PreviousKeyboardState = CurrentKeyboardState;
        }

        public void AddKeyboardInput(Keys theKey, bool isReleasedPreviously)
        {
            if (mKeyboard.ContainsKey(theKey))
            {
                return;
            }
            mKeyboard.Add(theKey, isReleasedPreviously);
        }

        public void AddGamepadInput(Buttons theButton, bool isReleasedPreviously)
        {
            if (mGamepad.ContainsKey(theButton))
            {
                return;
            }
            mGamepad.Add(theButton, isReleasedPreviously);
        }

        public bool IsPressed(PlayerIndex thePlayerIndex)
        {
            //Check to see if any of the keys associated with this Input
            //have been pressed
            foreach (Keys aKey in mKeyboard.Keys)
            {
                if (mKeyboard[aKey] == true)
                {
                    if (CurrentKeyboardState.IsKeyDown(aKey) == true && PreviousKeyboardState.IsKeyDown(aKey) == false)
                    {
                        return true;
                    }
                }
                else
                {
                    if (CurrentKeyboardState.IsKeyDown(aKey))
                    {
                        return true;
                    }
                }
            }

            //Check to see if any of the gamepad buttons associated with this Input
            //have been pressed
            foreach (Buttons aButton in mGamepad.Keys)
            {
                if (mGamepad[aButton] == true)
                {
                    if (CurrentGamePadState[thePlayerIndex].IsButtonDown(aButton) == true && PreviousGamePadState[thePlayerIndex].IsButtonDown(aButton) == false)
                    {
                        return true;
                    }
                }
                else
                {
                    if (CurrentGamePadState[thePlayerIndex].IsButtonDown(aButton))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

    }
}

